import { type CompilableTransactionMessage } from "@solana/transaction-messages";
import type { GetLatestBlockhashApi, Rpc, SimulateTransactionApi } from "@solana/rpc";
export type PrepareTransactionConfig = {
    /**
     * RPC client capable of simulating transactions and getting the latest blockhash
     **/
    rpc: Rpc<SimulateTransactionApi & GetLatestBlockhashApi>;
    /**
     * Multiplier applied to the simulated compute unit value obtained from simulation
     *
     * Default: `1.1`
     **/
    computeUnitLimitMultiplier?: number;
    /**
     * Whether or not you wish to force reset the compute unit limit value (if one is already set)
     * using the simulation response and `computeUnitLimitMultiplier`
     **/
    computeUnitLimitReset?: boolean;
    /**
     * Whether or not you wish to force reset the latest blockhash (if one is already set)
     *
     * Default: `true`
     **/
    blockhashReset?: boolean;
};
/**
 * Prepare a Transaction to be signed and sent to the network. Including:
 * - setting a compute unit limit (if not already set)
 * - fetching the latest blockhash (if not already set)
 * - (optional) simulating and resetting the compute unit limit
 * - (optional) resetting latest blockhash to the most recent
 */
export declare function prepareTransaction<TMessage extends CompilableTransactionMessage = CompilableTransactionMessage>(transaction: TMessage, config: PrepareTransactionConfig): Promise<TMessage>;
//# sourceMappingURL=prepare-transaction.d.ts.map